@@ -210,7 +210,7 @@ static void dump_vcis(unsigned long *base)
 {
     int i;
 
-    printf("VCI 0x%p\n",base);
+    printf("VCI %p\n",base);
     for (i = 0; i < NR_VCI; i++)
 	if ((base[i*4] >> MID_VCI_MODE_SHIFT) != MID_MODE_TRASH) {
 	    printf("VCI %d\n",i);
@@ -223,7 +223,7 @@ static void dump_rx_dma(unsigned long *base)
 {
     int i;
 
-    printf("RX DMA 0x%p\n",base);
+    printf("RX DMA %p\n",base);
     for (i = 0; i < NR_DMA_RX; i++) {
 	printf("RX DMA %d\n",i);
 	process(base+i*2,rx_dma);
@@ -235,7 +235,7 @@ static void dump_tx_dma(unsigned long *base)
 {
     int i;
 
-    printf("TX DMA 0x%p\n",base);
+    printf("TX DMA %p\n",base);
     for (i = 0; i < NR_DMA_TX; i++) {
 	printf("TX DMA %d\n",i);
 	process(base+i*2,tx_dma);
@@ -245,7 +245,7 @@ static void dump_tx_dma(unsigned long *base)
 
 static void dump_service(unsigned long *base)
 {
-    printf("SERV 0x%p\n",base);
+    printf("SERV %p\n",base);
 }
 
 
