@@ -32,14 +32,17 @@
 AsnOid atmfPortIndex;
 AsnOid atmfMyIpNmAddress;
 AsnOid atmfNetPrefixStatus;
+AsnOid atmfAtmLayerUniVersion;
 
 AsnInt atmfPortIndexValue;
-AsnOid atmfMyIpNmAddressValue;
+IpAddress atmfMyIpNmAddressValue;
+AsnInt atmfAtmLayerUniVersionValue;
 
 AsnOid *accessNetPrefix(void);
 void deleteNetPrefix(void);
 AsnInt getNetPrefix(VarBind *varbind, Variable *var);
 AsnInt getnextNetPrefix(VarBind *varbind, Variable *var);
 AsnInt setNetPrefix(VarBind *varbind, Variable *var);
+AsnInt getMyIp(VarBind *varbind, Variable *var);
 
 #endif
