@@ -57,6 +57,9 @@ void ilmi_loop(int fd, int itf);
 void usage(char *name)
 {
   fprintf(stderr, "usage: %s [-b] [-d] [-v] [-l logfile] [-x] [-q qos] "
+#ifdef DYNAMIC_UNI
+    "[ -u uni ] "
+#endif
     "<interface>\n", name);
   exit(1);
 }
@@ -290,11 +293,20 @@ int main(int argc, char *argv[])
       case 'x':
 	no_var_bindings = 1;
 	break;
+#ifdef DYNAMIC_UNI
+      case 'u':
+	if (!strcmp(optarg,"3.0") || !strcmp(optarg,"30"))
+	  atmfAtmLayerUniVersionValue = 2; // version3point0(2)
+	else if (!strcmp(optarg,"3.1") || !strcmp(optarg,"31"))
+	  atmfAtmLayerUniVersionValue = 3; // version3point1(3)
+	else if (!strcmp(optarg,"4.0") || !strcmp(optarg,"40"))
+	  atmfAtmLayerUniVersionValue = 4; // version4point0(4)
+	else usage(argv[0]);
+#endif
       case 'h':
       default:
 	usage(argv[0]);
       }
-
   if(argc == optind + 1)
     itf = atoi(argv[optind++]);
 
