@@ -31,7 +31,7 @@
 
 #define MAX_ILMI_MSG 484
 
-AsnOid *get_esi(int fd);
+AsnOid *get_esi(int fd, int itf);
 void update_nsap(int itf, AsnOid *netprefix, AsnOid *esi);
 int wait_for_message(int fd, struct timeval *timeout);
 int read_message(int fd, Message *message);
