@@ -323,25 +323,25 @@ static uint32_t get_local_ip(void)
 AsnInt getIpAddr(VarBind *varbind, Variable *var)
 {
   if (!((IpAddress*) var->value)->octs) {/* not set */
+      unsigned char *p;
       uint32_t ip;
+
+      ((IpAddress*) var->value)->octs = p = alloc(4);
       ip = get_local_ip();
-      if (ip) {
-	((IpAddress*) var->value)->octs = alloc(4);
-	*((uint32_t *) ((IpAddress*) var->value)->octs) = ip;
-      }
-  }
-  if (!((IpAddress*) var->value)->octs) /* still not set */
-    {
-      struct hostent* h;
-      char hostname[128];
-      ((IpAddress*) var->value)->octs = alloc(4);
-      if (!gethostname(hostname, sizeof(hostname)-1))
-	{
-	  h = (struct hostent*) gethostbyname(hostname);
-	  *((uint32_t *) ((IpAddress*) var->value)->octs) =
-	    *((uint32_t *) h->h_addr_list[0]);
+      if (ip) *((uint32_t *) p) = ip;
+      else {
+	struct hostent* h;
+	char hostname[128];
+
+	if (!gethostname(hostname, sizeof(hostname)-1)) {
+	    h = (struct hostent*) gethostbyname(hostname);
+	    if (!h) memset(p,4,0);
+	    else memcpy(p,h->h_addr_list[0],4);
 	}
-    };
+      }
+      diag(COMPONENT,DIAG_DEBUG,"Local IP address is %d.%d.%d.%d",p[0],p[1],
+	p[2],p[3]);
+  }
   varbind->value = Asn1Alloc(sizeof(ObjectSyntax));
   varbind->value->choiceId = OBJECTSYNTAX_APPLICATION_WIDE;
   varbind->value->a.application_wide = Asn1Alloc(sizeof(ApplicationSyntax));
