@@ -123,7 +123,7 @@ int un_recv_connect(int s,void *buf,int size);
  * address. Returns a negative value on error.
  */
 
-int un_reply(int s,void *buf,int _size,
+int un_reply(int s,void *buf,int size,
   int (*handler)(void *buf,int len,void *user),void *user);
 
 /*
@@ -132,7 +132,7 @@ int un_reply(int s,void *buf,int _size,
  * handler returns a negative value or zero, no reply is sent. If the handler
  * returns a positive value, this is interpreted as the length of the reply to
  * send. The data is taken from the buffer. If any system call fails, un_reply
- * returns a negative value. Otherwise, is returns whatever was returned by the
+ * returns a negative value. Otherwise, it returns whatever was returned by the
  * handler function.
  */
 
