@@ -41,7 +41,7 @@ _syscall3(int,poll,struct pollfd *,ufds,unsigned int,nfds,int,timeout);
 #define ddprintf(format,args...)
 #endif                                                                      
 
-extern int INTERFACE_NUMBER; /* from main.c */
+extern struct mpc_control mpc_control; /* from main.c */
 
 int keep_alive_sm_running = 0;
 
@@ -57,20 +57,12 @@ static struct llc_snap_hdr llc_snap_mpoa_ctrl = {
 	{0x00, 0x03}
 };        
 
-extern int kernel_socket;        /* kernel socket for mpcd <--> kernel msgs, from main.c  */
-extern int MPS_socket;           /* MPS control socket for out going msgs, from main.c     */
-extern int MPS_listen_socket;    /* MPS control listen socket, from main.c                 */
-extern unsigned char MPS_CTRL_ATM_ADDR[];
-extern unsigned char OWN_ATM_ADDRESS[];
-extern struct sockaddr_atmsvc data_listen_addr;
-
-extern int mps_ctrl_addr_set;    
-static time_t stay_alive;        /* Next Keep-Alive should come before we hit this time    */
+static time_t stay_alive;        /* Next Keep-Alive should come before we hit this time     */
 static struct pollfd fds[OPEN_MAX];
-static int first_empty;          /* first non-reserved slot in fds[]                       */
-static int fds_used;             /* first non-occupied slot in fds[]                       */
-static short socket_type[OPEN_MAX]; /* type and state info for fds[], see "io.h" for types */
-static struct outgoing_shortcut ingress_shortcuts[OPEN_MAX]; /* array of shortcuts we made */
+static int first_empty;          /* first non-reserved slot in fds[]                        */
+static int fds_used;             /* first non-occupied slot in fds[], => also # of used fds */
+static short socket_type[OPEN_MAX]; /* type and state info for fds[], see "io.h" for types  */
+static struct outgoing_shortcut ingress_shortcuts[OPEN_MAX]; /* array of shortcuts we made  */
 
 static int update_ingress_entry(uint32_t *addr, int fd, int new_state);
 static int msg_from_mps(int slot);
@@ -91,18 +83,18 @@ void main_loop(int listen_socket)
 	for (i = 0; i < OPEN_MAX; i++)
                 fds[i].fd = -1;
 
-        fds[0].fd     = kernel_socket;     /* mpcd <--> kernel socket   */
+        fds[0].fd     = mpc_control.kernel_socket;     /* mpcd <--> kernel socket   */
         fds[0].events = POLLIN;
         socket_type[0]= KERNEL;
 
-	if(!mps_ctrl_addr_set)             /* Can't do much without the MPS control ATM addr */
+	if(!mpc_control.mps_ctrl_addr_set)             /* Can't do much without the MPS control ATM addr */
 	        wait_for_mps_ctrl_addr();
 	connect_to_MPS();
 	
-        fds[1].fd     = MPS_socket;        /* we opened this to MPS      */
+        fds[1].fd     = mpc_control.MPS_socket;        /* we opened this to MPS      */
         fds[1].events = POLLIN;
         socket_type[1]= (OUTGOING_CTRL | CONNECTED);
-        fds[2].fd     = MPS_listen_socket; /* for incoming control calls */
+        fds[2].fd     = mpc_control.MPS_listen_socket; /* for incoming control calls */
         fds[2].events = POLLIN;
         socket_type[2]= LISTENING_CTRL;
         fds[3].fd     = listen_socket;     /* for incoming shortcuts     */
@@ -114,15 +106,23 @@ void main_loop(int listen_socket)
         while (1) {
                 kernel_ok = mps_ok = new_ctrl = new_shortcut = 1;
                 fflush(stdout);
-                changed_fds = poll(fds, OPEN_MAX, poll_timeout);
+                changed_fds = poll(fds, fds_used, poll_timeout);
 #if 0
-                changed_fds = poll(fds, fds_used + 1, poll_timeout);
+                printf("\nio.c: main_loop() poll returns %d\n", changed_fds);
+                for (i = 0; i < OPEN_MAX; i++) {
+                        if (fds[i].fd < 0 ) continue;    /* slot not in use */
+                        if ( fds[i].revents == 0) {
+                                printf("check1: fd %d slot %d not changed\n", fds[i].fd, i);
+                        }
+                        else printf("check1: fd %d slot %d     changed\n", fds[i].fd, i);
+                }
 #endif
+
                 switch(changed_fds) {
                 case -1:
                         printf("mpcd: io.c: main_loop: poll error: %s\n", strerror(errno));
 			if(errno == EINTR) continue;
-                        exit (1);
+                        return; /* return to main() */
                         break; /* not reached */
                 case 0:
 			keep_alive_sm(0, -1);  /* (keepalive_liftime, seq_num) */
@@ -140,53 +140,67 @@ void main_loop(int listen_socket)
 
                 /* Since we are here something happened to the fds */
                 if (fds[0].revents) {
-                        kernel_ok = msg_from_kernel(fds[0].fd);
                         dprintf("mpcd: io.c: main_loop() msg_from_kernel\n");
-                        --changed_fds;
+                        kernel_ok = msg_from_kernel(fds[0].fd);
+                        changed_fds--;
                 }
                 if (fds[1].revents) {
-                        mps_ok = msg_from_mps(1);
                         ddprintf("mpcd: io.c: main_loop() msg_from_mps1\n");
-                        --changed_fds;
+                        mps_ok = msg_from_mps(1);
+                        changed_fds--;
                 }
                 if (fds[2].revents) {
                         new_ctrl = accept_conn(2);
-                        socket_type[new_ctrl] = INCOMING_CTRL | CONNECTED;
-                        dprintf("mpcd: io.c main:_loop() accepted INCOMING_CTRL fd =%d\n", new_ctrl);
-                        --changed_fds;
-                }
+			changed_fds--;
+			if( new_ctrl < 0 )
+			        break;
+			socket_type[new_ctrl] = INCOMING_CTRL | CONNECTED;
+                        dprintf("mpcd: io.c main_loop() accepted INCOMING_CTRL slot %d\n", new_ctrl);
+		}
                 if (fds[3].revents) {
                         new_shortcut = accept_conn(3);
-                        dprintf("mpcd: io.c main:_loop() accepted INCOMING_SHORTCUT fd =%d\n", new_shortcut);
-                        socket_type[new_shortcut] = INCOMING_SHORTCUT;
+                        dprintf("mpcd: io.c main_loop() accepted INCOMING_SHORTCUT slot %d\n", new_shortcut);
+			changed_fds--;
+			if( new_shortcut < 0 )
+			        break;
+			socket_type[new_shortcut] = INCOMING_SHORTCUT;
                         if (add_shortcut(new_shortcut, MPC_SOCKET_EGRESS) < 0)
                                 break;
-                        --changed_fds;
-                }
+		}
 
+#if 0
                 if (changed_fds == 0)  /* see if we can already go back to poll() */
                         continue;
+#endif
 
-                for (i = first_empty; i <= fds_used; i++) {
-                        if (fds[i].fd < 0 || fds[i].revents == 0)
-                                continue;                /* not used or unchanged */
-                        if (socket_type[i] & INCOMING_CTRL) {
-                                mps_ok = msg_from_mps(i);
-                                ddprintf("mpcd: io.c: main_loop() msg_from_mps2\n");
+                for (i = first_empty; i < fds_used; i++) {
+                        if (fds[i].fd < 0 ) continue;    /* slot not in use */
+                        if ( fds[i].revents == 0) {
+                                ddprintf("fd %d slot %d not changed\n", fds[i].fd, i);
+                                continue;
                         }
-                        else if (check_connection(i) < 0)
-                                break; /* this will cause break from while(1) too */
-                        if (--changed_fds == 0)
-                                break; /* leave for() */
+                        ddprintf("about to process fd %d slot %d\n", fds[i].fd, i);
+                        if (socket_type[i] & INCOMING_CTRL) {
+                                ddprintf("mpcd: io.c: main_loop() msg_from_mps2\n");
+                                mps_ok = msg_from_mps(i);
+                        }
+                        else {
+                                ddprintf("mpcd: io.c: main_loop() checking connection fd %d\n", fds[i].fd);
+                                if (check_connection(i) < 0) {
+                                        printf("mpcd: io.c: main_loop: check_connection returned < 0\n");
+                                        break; /* this will cause break from while(1) too */
+                                }
+                        }
+			if (--changed_fds == 0) break; /* no more changed fds, leave for() */
                 }
 
                 if (changed_fds != 0){
-			dprintf("mpcd: changed_fds = %d\n", changed_fds);
-                        break;         /* leave while(1) */
+			printf("mpcd: changed_fds = %d\n", changed_fds);
+                        /* break; */         /* leave while(1) */
 		}
                 if (kernel_ok && mps_ok >= 0 && new_ctrl >= 0 && new_shortcut >= 0)
-                        continue; /* back to poll() */
-		//	break;
+		        continue; /* back to poll() */
+			
         }
         
         /* clean up, close the sockets */
@@ -197,6 +211,7 @@ void main_loop(int listen_socket)
                 socket_type[i] = NOT_USED;
         }
         printf("mpcd: io.c: exiting main_loop()\n");
+
         return;
 }
 
@@ -205,7 +220,7 @@ void main_loop(int listen_socket)
  * kernel has found one from a TLV in le_arp and tells us what it is.
  */
 static void wait_for_mps_ctrl_addr(){
-	while(!mps_ctrl_addr_set){
+	while(!mpc_control.mps_ctrl_addr_set){
 	        if(poll(fds, 1, -1))
 		        msg_from_kernel(fds[0].fd);
 	}
@@ -223,21 +238,22 @@ int send_to_mps(char *buff, int length)
         char tmp[MAX_PACKET_LENGTH + sizeof(struct llc_snap_hdr)];
 	int bytes_written;
         char *pos = tmp;
-	if(MPS_socket<0){
+	if(mpc_control.MPS_socket<0){
 	        connect_to_MPS();     
-		fds[1].fd = MPS_socket;
-		fds[1].events = POLLIN;
+		fds[1].fd = mpc_control.MPS_socket;
+		fds[1].events  = POLLIN;
+		fds[1].revents = 0;
 		socket_type[1] = (OUTGOING_CTRL | CONNECTED);
 	}
         memcpy(pos, &llc_snap_mpoa_ctrl, sizeof(struct llc_snap_hdr));
 	pos += sizeof(struct llc_snap_hdr);
 	memcpy(pos, buff, length);
 	length += sizeof(struct llc_snap_hdr);
-        bytes_written = write(MPS_socket, tmp, length);
+        bytes_written = write(mpc_control.MPS_socket, tmp, length);
 	while(bytes_written != 0){
-		bytes_written = write(MPS_socket, tmp+bytes_written, length-bytes_written);
+		bytes_written = write(mpc_control.MPS_socket, tmp+bytes_written, length-bytes_written);
 		if( bytes_written < 0 ){
-			printf("mpcd: io.c: write failed.");
+			printf("mpcd: io.c: send_to_mps() write failed\n");
 			return -1;
 		}
 	}
@@ -260,7 +276,7 @@ int send_to_dataplane(char *buff, int length, int shortcut_fd)
 	while(bytes_written != 0){
 		bytes_written = write(shortcut_fd, tmp+bytes_written, length-bytes_written);
 		if( bytes_written < 0 ){
-			printf("mpcd: io.c: write to dataplane failed.");
+			printf("mpcd: io.c: write to dataplane failed\n");
 			return -1;
 		}
 	}
@@ -292,7 +308,7 @@ void keep_alive_sm(unsigned keep_alive_lifetime, int sequence_number){
 	if( now > stay_alive ){
 	        dprintf("mpcd: io.c: MPS death!");
 	        msg.type = MPS_DEATH;
-		memcpy(msg.MPS_ctrl,MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
+		memcpy(msg.MPS_ctrl,mpc_control.MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
 		send_to_kernel(&msg);
 		previous_sequence_number = 0;
 		stay_alive = now + MPC_C2; 
@@ -303,7 +319,7 @@ void keep_alive_sm(unsigned keep_alive_lifetime, int sequence_number){
 	if( sequence_number < previous_sequence_number ){
 	         dprintf("mpcd: io.c: MPS death!");
 	         msg.type = MPS_DEATH;
-		 memcpy(msg.MPS_ctrl,MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
+		 memcpy(msg.MPS_ctrl,mpc_control.MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
 		 send_to_kernel(&msg);
 		 previous_sequence_number = 0;
 		 stay_alive = now + MPC_C2; 
@@ -323,40 +339,86 @@ void keep_alive_sm(unsigned keep_alive_lifetime, int sequence_number){
 int get_socket(struct sockaddr_atmsvc *address)
 {
         struct atm_qos qos;
-	struct atm_sap sap;
-	int socket_fd;
+        struct atm_sap sap;
+        int socket_fd;
 
         socket_fd = socket(PF_ATMSVC, SOCK_DGRAM, 0);
-	if( socket_fd < 0 ){
-	  printf("mpcd: io.c: socket creation failure: %s \n",strerror(errno));
-	  return -1;
-	}
-	memset(&qos, 0, sizeof(qos));
-	memset(&sap, 0, sizeof(sap));
-	qos.aal = ATM_AAL5;
-	qos.txtp.traffic_class = ATM_UBR;
-	qos.rxtp.traffic_class = ATM_UBR;
-	qos.txtp.max_sdu = 1536;
-	qos.rxtp.max_sdu = 1536; 
-	sap.blli[0].l2_proto = ATM_L2_ISO8802; 
-	if (setsockopt(socket_fd, SOL_ATM,SO_ATMQOS, &qos, sizeof(qos)) < 0){
-		printf("mpcd: io.c: setsockopt SO_ATMQOS failed: %s \n",strerror(errno));
-		close(socket_fd);
+        if( socket_fd < 0 ){
+          printf("mpcd: io.c: socket creation failure: %s \n",strerror(errno));
+          return -1;
+        }
+        memset(&qos, 0, sizeof(qos));
+        memset(&sap, 0, sizeof(sap));
+        qos.aal = ATM_AAL5;
+        qos.txtp.traffic_class = ATM_UBR;
+        qos.rxtp.traffic_class = ATM_UBR;
+        qos.txtp.max_sdu = 1536;
+        qos.rxtp.max_sdu = 1536;
+        sap.blli[0].l2_proto = ATM_L2_ISO8802;
+        if (setsockopt(socket_fd, SOL_ATM,SO_ATMQOS, &qos, sizeof(qos)) < 0){
+                printf("mpcd: io.c: setsockopt SO_ATMQOS failed: %s \n",strerror(errno));
+                close(socket_fd);                      
 		return -1;
-	}
-	if (setsockopt(socket_fd,SOL_ATM,SO_ATMSAP,&sap,sizeof(sap)) < 0) {
-		printf("mpcd: io.c: setsockop SO_ATMSAP failed\n");
-		close (socket_fd);
+        }
+        if (setsockopt(socket_fd,SOL_ATM,SO_ATMSAP,&sap,sizeof(sap)) < 0) {
+                printf("mpcd: io.c: setsockop SO_ATMSAP failed\n");
+                close (socket_fd);
+                return -1;
+        }
+        if (address == NULL)
+                return socket_fd;
+
+        if (bind(socket_fd, (struct sockaddr *)address, sizeof(struct sockaddr_atmsvc)) < 0){
+                printf("mpcd: io.c: bind  failure: %s \n",strerror(errno));
+                close(socket_fd);
+                return -1;
+        }
+        return socket_fd;
+}                          	
+
+/*
+ * Creates an ATM_ANYCLASS traffic class socket, sets traffic and sap
+ * parameters and binds the socket with given address.
+ *
+ * returns < 0 for error, socket for ok
+ */
+int get_listen_socket(struct sockaddr_atmsvc *address)
+{
+
+        int s;
+        struct atm_qos qos;
+
+        s = get_socket(NULL);
+        if (s < 0) {
+                printf("mpcd: io.c: get_listen_socket() get socket failed\n");
+                return s;
+        }
+
+        memset(&qos, 0, sizeof(qos));
+        qos.aal = ATM_AAL5;
+        qos.txtp.traffic_class = ATM_ANYCLASS;
+        qos.rxtp.traffic_class = ATM_ANYCLASS;
+
+        if (setsockopt(s, SOL_ATM,SO_ATMQOS, &qos, sizeof(qos)) < 0){
+                printf("mpcd: io.c: get_listen_socket() setsockopt SO_ATMQOS: %s\n",
+                       strerror(errno));
+                close(s);                      
 		return -1;
-	}
-	if (address == NULL)
-		return socket_fd;
-	if (bind(socket_fd, (struct sockaddr *)address, sizeof(struct sockaddr_atmsvc)) < 0){
-		printf("mpcd: io.c: bind  failure: %s \n",strerror(errno));
-		close(socket_fd);
-		return -1;  
-	}
-	return socket_fd;
+        }
+
+        if (bind(s, (struct sockaddr *)address, sizeof(struct sockaddr_atmsvc)) < 0){
+                printf("mpcd: io.c: get_listen_socket() bind: %s\n", strerror(errno));
+                close(s);
+                return -1;
+        }
+        
+        if (listen(s, 5) < 0) {
+                printf("mpcd: io.c: get_lilsten_socket() listen: %s\n", strerror(errno));
+                close(s);
+                return -1;
+        }
+
+        return s;
 }
 
 /*
@@ -415,7 +477,7 @@ static int msg_from_mps(int slot)
                 close(fd);
                 fds[slot].fd = -1;
                 if (slot == 1)
-		        MPS_socket = -1;
+		        mpc_control.MPS_socket = -1;
                 socket_type[slot] = NOT_USED;
                 return 1; /* See spec 4.6. Might be normal */
         }
@@ -449,17 +511,18 @@ static int accept_conn(int slot)
         for (i = first_empty; i < OPEN_MAX; i++) {
                 if (fds[i].fd >= 0)                /* slot in use ? */
                         continue;
-                fds[i].fd     = new_fd;
-                fds[i].events = POLLIN;
+                fds[i].fd      = new_fd;
+                fds[i].events  = POLLIN;
+                fds[i].revents = 0;
                 break;
         }
         if (i == OPEN_MAX) {
-                printf("mpcd: io.c: do_accept: no room for new connection\n");
+                printf("mpcd: io.c: accept_conn: no room for new connection\n");
                 return -1;
         }
 
-        if (i > fds_used)
-                fds_used = i;
+        if (i >= fds_used)
+                fds_used = i + 1;
 
         return i;
 }
@@ -476,7 +539,7 @@ static int add_shortcut(int slot, int type)
         if (type == MPC_SOCKET_INGRESS)
                 ipaddr = update_ingress_entry(NULL, fds[slot].fd, INGRESS_CONNECTED);
 
-        ioc_data.dev_num = INTERFACE_NUMBER;
+        ioc_data.dev_num = mpc_control.INTERFACE_NUMBER;
         ioc_data.ipaddr  = ipaddr;
         ioc_data.type    = type;
         if (ioctl(fds[slot].fd, ATMMPC_DATA, &ioc_data) < 0) {
@@ -504,10 +567,12 @@ static int check_connection(int slot)
         int bytes_read;
 
         dprintf("mpcd: io.c: check_connection() event in fd %d, type %d\n", fds[slot].fd, socket_type[slot]);
+        if (socket_type[slot] & CONNECTING) { /* connect() completed (maybe) */
+                complete_connection(slot);    /* ignore return value */
+                return 0;
+        }
+
         pfd = &fds[slot];
-        if (socket_type[slot] & CONNECTING) /* connect() completed (maybe) */
-                return complete_connection(slot);
-        
         bytes_read = read(pfd->fd, buff, sizeof(buff));
         if (bytes_read < 0) {
                 if (errno == ECONNRESET || errno == EPIPE) { /* conn reset by the other end or kernel (EPIPE) */
@@ -542,7 +607,7 @@ static int check_connection(int slot)
 	msg = (struct k_message *)buff;
 	if(msg->type == DATA_PLANE_PURGE){
 	        send_purge_request(msg->content.eg_info.mps_ip,32,
-				   get_own_ip_addr(INTERFACE_NUMBER),pfd->fd);
+				   get_own_ip_addr(mpc_control.INTERFACE_NUMBER),pfd->fd);
 		return 1;
 	}
 
@@ -553,7 +618,7 @@ static int check_connection(int slot)
 }
 
 /*
- * returns < 0 for error, socket for ok
+ * returns < 0 for unsuccessful connect, fd for ok
  *
  */
 static int complete_connection(int slot)
@@ -567,12 +632,20 @@ static int complete_connection(int slot)
          * completed successfully
          */
         retval = connect(fds[slot].fd,(struct sockaddr *)&dummy, sizeof(dummy));
-        if (retval != 0)
-                return -1;
+        if (retval < 0) {
+                printf("mpcd: io.c: complete_connection(): '%s'\n", strerror(errno));
+                socket_type[slot] = NOT_USED;
+                update_ingress_entry(NULL, fds[slot].fd, INGRESS_NOT_USED);
+                close(fds[slot].fd);
+                fds[slot].fd = -1;
+                fds[slot].revents = 0;
+                return 0;
+        }
         socket_type[slot] &= ~CONNECTING;
         socket_type[slot] |= CONNECTED;
         
-        fds[slot].events = POLLIN; /* We left POLLOUT accidentally in. Hope you never do the same */
+        fds[slot].events  = POLLIN; /* We left POLLOUT accidentally in. Hope you never do the same */
+        fds[slot].revents = 0;
         if(socket_type[slot] & OUTGOING_SHORTCUT)
                 return add_shortcut(slot, MPC_SOCKET_INGRESS);
         return fds[slot].fd;
@@ -594,8 +667,9 @@ void create_ingress_svc(uint32_t ipaddr, char *atm_addr, struct atm_qos qos)
         for (i = first_empty; i < OPEN_MAX; i++) {
 	        if (fds[i].fd >= 0)                /* slot in use ? */
                         continue;
-                fds[i].fd     = new_socket;
-                fds[i].events = POLLIN | POLLOUT;
+                fds[i].fd      = new_socket;
+                fds[i].events  = POLLIN | POLLOUT;
+                fds[i].revents = 0;
                 break;
         }
         if (i == OPEN_MAX) {
@@ -604,8 +678,8 @@ void create_ingress_svc(uint32_t ipaddr, char *atm_addr, struct atm_qos qos)
         }
 
         socket_type[i] = (OUTGOING_SHORTCUT | CONNECTING);
-        if (i > fds_used)
-                fds_used = i;
+        if (i >= fds_used)
+                fds_used = i + 1;
 
         /* Store the IP address we are creating this shortcut for */
         dprintf("mpcd: io.c: create_ingress_svc: adding ip 0x%x\n", ipaddr);
@@ -637,7 +711,7 @@ static int create_shortcut(char *atm_addr, struct atm_qos qos)
         memset(&addr, 0, sizeof(addr));
         addr.sas_family = AF_ATMSVC;
         memcpy(addr.sas_addr.prv, atm_addr, ATM_ESA_LEN);
-	s = get_socket(&data_listen_addr);
+	s = get_socket(&mpc_control.data_listen_addr);
 	if(qos.txtp.traffic_class > ATM_UBR || qos.rxtp.traffic_class > ATM_UBR){
 	        if (setsockopt(s, SOL_ATM,SO_ATMQOS, &qos, sizeof(qos)) < 0){
 		        printf("mpcd: io.c: setsockopt SO_ATMQOS failed: %s \n",strerror(errno));
@@ -677,18 +751,18 @@ static int connect_to_MPS(){
 	struct sockaddr_atmsvc ctrl_listen_addr;
 	memset(&mps_ctrl_addr,0,sizeof(struct sockaddr_atmsvc));
 	memset(&ctrl_listen_addr,0,sizeof(struct sockaddr_atmsvc));
-	memcpy(mps_ctrl_addr.sas_addr.prv,MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
-	memcpy(ctrl_listen_addr.sas_addr.prv,OWN_ATM_ADDRESS,ATM_ESA_LEN);
+	memcpy(mps_ctrl_addr.sas_addr.prv,mpc_control.MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
+	memcpy(ctrl_listen_addr.sas_addr.prv,mpc_control.OWN_ATM_ADDRESS,ATM_ESA_LEN);
 	mps_ctrl_addr.sas_family = AF_ATMSVC;
 	ctrl_listen_addr.sas_family = AF_ATMSVC;
-        MPS_socket = get_socket(&ctrl_listen_addr);
-        if (MPS_socket < 0)
+        mpc_control.MPS_socket = get_socket(&ctrl_listen_addr);
+        if (mpc_control.MPS_socket < 0)
                 return -1;
-	c = connect(MPS_socket, (struct sockaddr *)&(mps_ctrl_addr), 
+	c = connect(mpc_control.MPS_socket, (struct sockaddr *)&(mps_ctrl_addr), 
 		    sizeof(struct sockaddr_atmsvc));
         if( c < 0 ){
 		printf("mpcd: io.c: connect to MPS failed: %s \n",strerror(errno));
-		close(MPS_socket);
+		close(mpc_control.MPS_socket);
                 return -1;  
         }
 	return 0;
