@@ -43,41 +43,20 @@ static int send_setup(SOCKET *sock)
     while (walk);
     sock->call_ref = call_ref;
     q_create(&dsc,q_buffer,MAX_Q_MSG);
-    q_assign(&dsc,QF_call_ref,call_ref);
-    q_assign(&dsc,QF_msg_type,ATM_MSG_SETUP);
-    q_assign(&dsc,QF_aal_type,5); /* AAL 5 */
-#ifdef UNI30
-    q_assign(&dsc,QF_aal_mode,1); /* Message mode - LANE seems to really want
-				     this */
-#endif
-    q_assign(&dsc,QF_sscs_type,0); /* unspecified - LANE wants this */
-    error = sap_encode(&dsc,&sock->remote,&sock->sap,&sock->qos);
-    q_assign(&dsc,QF_bearer_class,16); /* BCOB-X */
-#if defined(UNI30) || defined(ALLOW_UNI30)
-    q_assign(&dsc,QF_trans_cap,ATM_TC_VBR_NRT_R00);
-	/* force presence - UNI 3.0 wants this */
-#endif
-    q_assign(&dsc,QF_upcc,0); /* p2p */
-#ifndef UNI30
-    q_assign(&dsc,QF_qos_cs,Q2931_CS_ITU);
-#endif
-    q_assign(&dsc,QF_qos_fw,0); /* QOS 0 */
-    q_assign(&dsc,QF_qos_bw,0);
     local = NULL;
+    error = 0;
     if (atmsvc_addr_in_use(sock->local)) local = &sock->local;
     else {
 	sock->sig = route_call(&sock->remote);
-	if (!sock->sig) {
-		if (!error) error = -EHOSTUNREACH;
-	}
+	if (!sock->sig) error = -EHOSTUNREACH;
 	else {
 	    local = get_local(sock->sig->signaling_pvc.sap_addr.itf);
 	    if (!local) local = get_local(ATM_ITF_ANY);
 	    if (local) sock->local = *local;
-	    else if (!error) {
-		    error = -EADDRNOTAVAIL;
-		    diag(COMPONENT,DIAG_ERROR,"no local address");
-		}
+	    else {
+		error = -EADDRNOTAVAIL;
+		diag(COMPONENT,DIAG_ERROR,"no local address");
+	    }
 	}
     }
     if (local)
@@ -92,6 +71,29 @@ static int send_setup(SOCKET *sock)
 	    q_assign(&dsc,QF_cgpn_type,ATM_TON_UNKNOWN);
 	    q_write(&dsc,QF_cgpn,(void *) local->sas_addr.prv,ATM_ESA_LEN);
 	}
+    q_assign(&dsc,QF_call_ref,call_ref);
+    q_assign(&dsc,QF_msg_type,ATM_MSG_SETUP);
+    q_assign(&dsc,QF_aal_type,5); /* AAL 5 */
+#if defined(UNI30) || defined(DYNAMIC_UNI)
+    if (sock->sig->uni == S_UNI30)
+	q_assign(&dsc,QF_aal_mode,1); /* Message mode - LANE seems to really
+					 want this */
+#endif
+    q_assign(&dsc,QF_sscs_type,0); /* unspecified - LANE wants this */
+    if (!error) error = sap_encode(&dsc,&sock->remote,&sock->sap,&sock->qos);
+    q_assign(&dsc,QF_bearer_class,16); /* BCOB-X */
+#if defined(UNI30) || defined(ALLOW_UNI30) || defined(DYNAMIC_UNI)
+    if (sock->sig->uni & S_UNI30)
+	q_assign(&dsc,QF_trans_cap,ATM_TC_VBR_NRT_R00);
+	  /* force presence - UNI 3.0 wants this */
+#endif
+    q_assign(&dsc,QF_upcc,0); /* p2p */
+#if !defined(UNI30) || defined(DYNAMIC_UNI)
+    if (sock->sig->uni != S_UNI30)
+	q_assign(&dsc,QF_qos_cs,Q2931_CS_ITU);
+#endif
+    q_assign(&dsc,QF_qos_fw,0); /* QOS 0 */
+    q_assign(&dsc,QF_qos_bw,0);
     if (sock->sig->mode != sm_user) {
 	if (!atmpvc_addr_in_use(sock->pvc)) {
 	    int vpci,vci;
@@ -128,9 +130,10 @@ static int send_connect(SOCKET *sock)
     if (sock->ep_ref >= 0) q_assign(&dsc,QF_ep_ref,sock->ep_ref);
     if (sock->ep_ref <= 0) { /* no AAL parameters if ep_ref present and != 0 */
 	q_assign(&dsc,QF_aal_type,5);
-#ifdef UNI30
-	q_assign(&dsc,QF_aal_mode,1); /* Message mode - LANE seems to really
-					 want this */
+#if defined(UNI30) || defined(DYNAMIC_UNI)
+	if (sock->sig->uni == S_UNI30)
+	    q_assign(&dsc,QF_aal_mode,1); /* Message mode - LANE seems to
+					     really want this */
 #endif
 	q_assign(&dsc,QF_sscs_type,0); /* unspecified - LANE wants this */
 	q_assign(&dsc,QF_fw_max_sdu,sock->qos.rxtp.max_sdu);
@@ -141,7 +144,7 @@ static int send_connect(SOCKET *sock)
 }
 
 
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 
 static void send_modify_request(SOCKET *sock)
 {
@@ -155,7 +158,7 @@ static void send_modify_request(SOCKET *sock)
 	q_assign(&dsc,QF_fw_pcr_01,SELECT_TOP_PCR(sock->new_qos.txtp));
     if (sock->new_qos.rxtp.traffic_class)
 	q_assign(&dsc,QF_bw_pcr_01,SELECT_TOP_PCR(sock->new_qos.rxtp));
-    if ((size = q_close(&dsc)) >= 0) to_signaling(sig,q_buffer,size);
+    if ((size = q_close(&dsc)) >= 0) to_signaling(sock->sig,q_buffer,size);
 }
 
 
@@ -172,7 +175,7 @@ static void send_modify_ack(SOCKET *sock)
 	q_assign(&dsc,QF_type_of_report,ATM_TOR_MOD_CONF);
 	START_TIMER(sock,T361);
     }
-    if ((size = q_close(&dsc)) >= 0) to_signaling(sig,q_buffer,size);
+    if ((size = q_close(&dsc)) >= 0) to_signaling(sock->sig,q_buffer,size);
 }
 
 #endif
@@ -324,12 +327,12 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		    return;
 		case ss_connecting:
 		case ss_accepting:
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 		case ss_mod_req:
 #endif
 		    STOP_TIMER(sock);
 		    /* fall through */
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 		case ss_mod_lcl:
 		case ss_mod_rcv:
 		case ss_mod_fin_ok:
@@ -337,7 +340,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		case ss_mod_fin_ack:
 #endif
 		case ss_connected:
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 		    if (timer_handler(sock->conn_timer) == on_T361)
 			STOP_TIMER(sock);
 #endif
@@ -345,10 +348,15 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 			diag(COMPONENT,DIAG_INFO,"Active close (CR 0x%06X)",
 			  sock->call_ref);
 		    send_release(sock,
+#ifdef DYNAMIC_UNI
+		      sock->sig->uni != S_UNI30 ? ATM_CV_NORMAL_CLEAR :
+		      ATM_CV_NORMAL_UNSPEC
+#else
 #if defined(UNI31) || defined(UNI40)
 		      ATM_CV_NORMAL_CLEAR
 #else
 		      ATM_CV_NORMAL_UNSPEC
+#endif
 #endif
 		      );
 		    START_TIMER(sock,T308_1);
@@ -377,8 +385,13 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		diag(COMPONENT,DIAG_FATAL,"s_c_p failed");
 	    new_state(sock,ss_proceeding);
 	    return;
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 	case as_modify:
+	    if (sock && !(sock->sig->uni & S_Q2963_1)) {
+		send_kernel(sock->id,0,as_okay,-ENOPROTOOPT,NULL,NULL,NULL,
+		  NULL,NULL);
+		return;
+	    }
 	    if (sock && (sock->state == ss_mod_lcl ||
 	      sock->state == ss_mod_req || sock->state == ss_mod_rcv ||
 	      sock->state == ss_mod_fin_ok || sock->state == ss_mod_fin_fail ||
